-- name: [CS] \\#ac50ff\\Kaktus
-- description: Kaktus arrives at Peach's Castle after taking a wrong turn on the I-35. \n\n\\#ff7777\\This Pack requires Character Select\nto use as a Library!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
	(This is an edited version of the Template File by Squishy)
]]

local TEXT_MOD_NAME = "[CS] Kaktus"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_KAKTUS = smlua_model_util_get_id("kaktusnew_geo") -- Located in "actors"
local E_MODEL_KAKTUSMAN = smlua_model_util_get_id("kaktus_geo") -- Located in "actors"
local E_MODEL_KAKTUSOLD = smlua_model_util_get_id("kaktusold_geo") -- Located in "actors"

local KAKTUS_ICON = get_texture_info("Kaktus_Icon") -- Located in "textures"
local KAKTUS_ICON_MALE = get_texture_info("Kaktus_Icon - Copy") -- Located in "textures"
local KAKTUS_ICON_OLD = get_texture_info("Kaktus_IconOLD")

-- All Located in "sound" Name them whatever you want. Remember to include the .ogg extension
local VOICETABLE_KAKTUS = {
    [CHAR_SOUND_OKEY_DOKEY] = 'Silent.ogg', -- Starting game
	[CHAR_SOUND_LETS_A_GO] = 'Silent.ogg', -- Starting level
	[CHAR_SOUND_PUNCH_YAH] = 'hit1.ogg', -- Punch 1
	[CHAR_SOUND_PUNCH_WAH] = 'hit2.ogg', -- Punch 2
	[CHAR_SOUND_PUNCH_HOO] = 'kick.ogg', -- Punch 3
	[CHAR_SOUND_YAH_WAH_HOO] = {'jump1.ogg', 'jump2.ogg', 'jump3.ogg'}, -- First/Second jump sounds
	[CHAR_SOUND_HOOHOO] = {'jump1.ogg', 'jump2.ogg', 'jump3.ogg'}, -- Third jump sound
	[CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'triplejump.ogg', 'longjump.ogg'}, -- Triple jump sounds
	[CHAR_SOUND_UH] = 'ohmygod.ogg', -- Wall bonk
	[CHAR_SOUND_UH2] = 'Silent.ogg', -- Landing after long jump
	[CHAR_SOUND_UH2_2] = 'longjumpland.ogg', -- Same sound as UH2; jumping onto ledge
	[CHAR_SOUND_HAHA] = 'landtriple.ogg', -- Landing triple jump
	[CHAR_SOUND_YAHOO] = 'longjump.ogg', -- Long jump
	[CHAR_SOUND_DOH] = 'wallhit.ogg', -- Long jump wall bonk
	[CHAR_SOUND_WHOA] = 'ohmygod.ogg', -- Grabbing ledge
	[CHAR_SOUND_EEUH] = 'lift.ogg', -- Climbing over ledge
	[CHAR_SOUND_WAAAOOOW] = 'fallong.ogg', -- Falling a long distance
	[CHAR_SOUND_TWIRL_BOUNCE] = 'holyshit.ogg', -- Bouncing off of a flower spring
	[CHAR_SOUND_GROUND_POUND_WAH] = 'Silent.ogg',
	[CHAR_SOUND_HRMM] = 'lift.ogg', -- Lifting something
	[CHAR_SOUND_HERE_WE_GO] = {'stargetnew.ogg', 'stargetnew2.ogg', 'stargetnew3.ogg', 'stargetnew5.ogg'}, -- Star get
	[CHAR_SOUND_SO_LONGA_BOWSER] = 'solonga.ogg', -- Throwing Bowser
--DAMAGE
	[CHAR_SOUND_ATTACKED] = 'hurt.ogg', -- Damaged
	[CHAR_SOUND_PANTING] = 'lowhealth.ogg', -- Low health
	[CHAR_SOUND_ON_FIRE] = 'assfire.ogg', -- Burned
--SLEEP SOUNDS
	[CHAR_SOUND_IMA_TIRED] = 'go.ogg', -- Mario feeling tired
	[CHAR_SOUND_YAWNING] = 'yawn.ogg', -- Mario yawning before he sits down to sleep
	[CHAR_SOUND_SNORING1] = 'Silent.ogg', -- Snore Inhale
	[CHAR_SOUND_SNORING2] = 'Silent.ogg', -- Exhale
	[CHAR_SOUND_SNORING3] = 'sleeptalking.ogg', -- Sleep talking / mumbling
--COUGHING (USED IN THE GAS MAZE)
	[CHAR_SOUND_COUGHING1] = 'cough1.ogg', -- Cough take 1
	[CHAR_SOUND_COUGHING2] = 'cough2.ogg', -- Cough take 2
	[CHAR_SOUND_COUGHING3] = 'cough3.ogg', -- Cough take 3
--DEATH
	[CHAR_SOUND_DYING] = 'deathdamage.ogg', -- Dying from damage
	[CHAR_SOUND_DROWNING] = 'drown.ogg', -- Running out of air underwater
	[CHAR_SOUND_MAMA_MIA] = 'levelboot.ogg', -- Booted out of level
}

-- All Located in "actors"
local CAPTABLE_CHAR = {
   normal = smlua_model_util_get_id("kaktus_caps_geo"),
   wing = smlua_model_util_get_id("kaktus_caps_wing_geo"),
   metal = smlua_model_util_get_id("kaktus_caps_metal_geo"),
   metalWing = smlua_model_util_get_id("kaktus_caps_metal_wing_geo"),
}

local ANIMTABLE_KAKTUS = {
    [CHAR_ANIM_IDLE_HEAD_CENTER] = 'idlingkak',
    [CHAR_ANIM_IDLE_HEAD_LEFT] = 'idlingkak',
    [CHAR_ANIM_IDLE_HEAD_RIGHT] = 'idlingkak',
    [CHAR_ANIM_TRIPLE_JUMP] = 'triplejump',
    [CHAR_ANIM_CROUCHING] = 'crouch',
    [CHAR_ANIM_CRAWLING] = 'crouchwalk',
    [CHAR_ANIM_START_CROUCHING] = 'crouchstart',
    [CHAR_ANIM_STOP_CROUCHING] = 'crouchstop',
    [CHAR_ANIM_START_CRAWLING] = 'crouchfreeze',
    [CHAR_ANIM_STOP_CRAWLING] = 'crouchfreeze'
}

local PALETTE_CHAR = {
    [PANTS]  = "242849",
    [SHIRT]  = "70157d",
    [GLOVES] = "FFB97B",
    [SHOES]  = "d9004d",
    [HAIR]   = "5e250d",
    [SKIN]   = "FFbb7e",
    [CAP]    = "30a33c",
	[EMBLEM] = "DF8424"
}

local PALETTE_MALE = {
    [PANTS]  = "DF8424",
    [SHIRT]  = "521a52",
    [GLOVES] = "ffffff",
    [SHOES]  = "242849",
    [HAIR]   = "5e250d",
    [SKIN]   = "FFbb7e",
    [CAP]    = "30a33c",
[EMBLEM] = "ac50ff"
}

local HM_KAKTUS= {
    label = {
        left = get_texture_info("KakLeftHealth"),
        right = get_texture_info("KakRightHealth"),
    },
    pie = {
        [1] = get_texture_info("KakPie1"),
        [2] = get_texture_info("KakPie2"),
        [3] = get_texture_info("KakPie3"),
        [4] = get_texture_info("KakPie4"),
        [5] = get_texture_info("KakPie5"),
        [6] = get_texture_info("KakPie6"),
        [7] = get_texture_info("KakPie7"),
        [8] = get_texture_info("KakPie8"),
    }
}

local CSloaded = false
local function on_character_select_load()
    CT_KAKTUS = _G.charSelect.character_add("Kaktus", {"Kaktus arrives at Peach's Castle", "after taking a wrong turn on the", "I-35.", "", "Custom Moveset Guide:", "Instead of diving, you dash", "forward and bounce off of walls.", "Press B while doing a ground", "pound to dive.", "You drown slightly faster."}, "ManIsSaul, Charlie & Kaktus64", {r = 172, g = 80, b = 255}, E_MODEL_KAKTUS, CT_KAKTUS, KAKTUS_ICON)
    _G.charSelect.character_add_caps(E_MODEL_KAKTUS, CAPTABLE_CHAR)
    _G.charSelect.character_add_voice(E_MODEL_KAKTUS, VOICETABLE_KAKTUS)
    _G.charSelect.character_add_celebration_star(E_MODEL_KAKTUS, E_MODEL_CUSTOM_STAR, TEX_CUSTOM_STAR_ICON)
    _G.charSelect.character_add_palette_preset(E_MODEL_KAKTUS, PALETTE_CHAR)
    _G.charSelect.character_add_health_meter(CT_KAKTUS, HM_KAKTUS)
    _G.charSelect.character_add_animations(E_MODEL_KAKTUS, ANIMTABLE_KAKTUS)
    _G.charSelect.character_hook_moveset(CT_KAKTUS, HOOK_MARIO_UPDATE, HOOK_BEFORE_SET_MARIO_ACTION)

    _G.charSelect.character_add_costume(CT_KAKTUS, "Kaktus (Old Outfit)", nil, "ManIsSaul, Charlie & Kaktus64", nil, E_MODEL_KAKTUSMAN, CT_WALUIGI, KAKTUS_ICON_MALE, nil, nil)
    _G.charSelect.character_add_voice(E_MODEL_KAKTUSMAN, VOICETABLE_KAKTUS)
    _G.charSelect.character_add_palette_preset(E_MODEL_KAKTUSMAN, PALETTE_MALE)
    _G.charSelect.character_add_caps(E_MODEL_KAKTUSMAN, CAPTABLE_CHAR)
    _G.charSelect.character_add_animations(E_MODEL_KAKTUSMAN, ANIMTABLE_KAKTUS)
    _G.charSelect.character_hook_moveset(CT_KAKTUS, HOOK_MARIO_UPDATE, HOOK_BEFORE_SET_MARIO_ACTION)

    _G.charSelect.character_add_costume(CT_KAKTUS, "Baldtus", nil, "ManIsSaul, Charlie & Kaktus64", nil, E_MODEL_KAKTUSOLD, CT_TOAD, KAKTUS_ICON_OLD, nil, nil)
    _G.charSelect.character_add_voice(E_MODEL_KAKTUSOLD, VOICETABLE_KAKTUS)
    _G.charSelect.character_add_palette_preset(E_MODEL_KAKTUSOLD, PALETTE_CHAR)
    _G.charSelect.character_add_caps(E_MODEL_KAKTUSOLD, CAPTABLE_CHAR)
    _G.charSelect.character_hook_moveset(CT_KAKTUS, HOOK_MARIO_UPDATE, HOOK_BEFORE_SET_MARIO_ACTION)
    add_moveset()
    CSloaded = true
end

local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_KAKTUS then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_KAKTUS then return _G.charSelect.voice.snore(m) end
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)

local function menupose(m)
    if _G.charSelect.is_menu_open() and
    CT_KAKTUS == _G.charSelect.character_get_current_number() and m.playerIndex == 0  then
        smlua_anim_util_set_animation(m.marioObj, "menupose")
        m.marioBodyState.handState = MARIO_HAND_PEACE_SIGN
        end
    end

    hook_event(HOOK_MARIO_UPDATE, menupose)